@@ -20,10 +20,6 @@ from zeep import Client
 from zeep.exceptions import Fault
 from zeep.wsse.username import UsernameToken
 
-import os
-certificates_dir = os.path.dirname(os.path.realpath(__file__)) + "/certificates/certificates.crt"
-os.environ['REQUESTS_CA_BUNDLE'] = certificates_dir
-
 
 class Connector(object):
 
@@ -83,6 +79,7 @@ class Connector(object):
 
         client_ucfe_requirement = client.type_factory('ns0').RequerimientoParaUcfe()
         client_ucfe_requirement.TipoMensaje = ucfe_requirement.message_type
+        client_ucfe_requirement.TipoNotificacion = ucfe_requirement.notification_type
         client_ucfe_requirement.CodComercio = ucfe_requirement.commerce_code
         client_ucfe_requirement.Serie = ucfe_requirement.serial
         client_ucfe_requirement.NumeroCfe = ucfe_requirement.cfe_number
