@@ -58,6 +58,9 @@ MESSAGES_TYPES = {
     'lot_response': '341',
     'state_request': '360',
     'state_response': '361',
+    'available_notification': '600',
+    'notification_data': '610',
+    'discard_notification': '620'
 }
 
 LOCAL_COUNTRY_CODE = 'UY'
@@ -90,7 +93,7 @@ def round_half_away_from_zero(n, digits=0):
 class Manager(object):
     __slots__ = ['emisor', 'comprobante', 'connection_data']
 
-    def __init__(self, emisor, comprobante, connection_data):
+    def __init__(self, connection_data, comprobante=None, emisor=None):
         self.emisor = emisor
         self.comprobante = comprobante
         self.connection_data = connection_data
@@ -120,6 +123,15 @@ class Manager(object):
         UcfeReq.cfe_xml_or_text = xml
         return UcfeReq
 
+    def create_notification_ucfe_req(self, message_type, notification_type=None, id_req=None):
+        UcfeReq = ucfe_requirement.UcfeRequirement()
+        UcfeReq.message_type = message_type
+        UcfeReq.notification_type = notification_type
+        UcfeReq.commerce_code = self.connection_data.commerce_code
+        UcfeReq.terminal_code = self.connection_data.terminal_code
+        UcfeReq.id_req = id_req
+        return UcfeReq
+
     def __send(self, ReqBody, UcfeReq):
         """
         Envia a webservice los datos que se encuentren en reqBody y ucfeReq,
@@ -145,6 +157,54 @@ class Manager(object):
 
         return self.__send(ReqBody, UcfeReq)
 
+    def get_available_notification(self, notification_type=None):
+        """Obtiene la primera notificación disponible de la cola
+
+        :param notification_type: Tipo de notificación que se quiere pedir, 
+        por defecto None en caso que se quiera cualquiera
+        :type notification_type: string, optional
+        :return: Mensaje de respuesta, cuerpo de respuesta y request
+        :rtype: dict
+        """        
+
+        ReqBody = self.create_req_body()
+        UcfeReq = self.create_notification_ucfe_req(MESSAGES_TYPES.get('available_notification'),
+                                                    notification_type)
+
+        return self.__send(ReqBody, UcfeReq)
+
+    def get_notification_data(self, id_req):
+        """Obtiene la información de una notificación
+
+        :param id_req: Id de la notificación a consultar
+        :type id_req: string
+        :return: Mensaje de respuesta, cuerpo de respuesta y request
+        :rtype: dict
+        """ 
+
+        ReqBody = self.create_req_body()
+        UcfeReq = self.create_notification_ucfe_req(message_type=MESSAGES_TYPES.get('notification_data'), 
+                                                    id_req=id_req)
+
+        return self.__send(ReqBody, UcfeReq)
+    
+    def discard_notification(self, notification_type, id_req):
+        """Descarta una notificación
+
+        :param notification_type: Tipo de la notificación
+        :type notification_type: string
+        :param id_req: Id de la notificación a descartar
+        :type id_req: string
+        :return: Mensaje de respuesta, cuerpo de respuesta y request
+        :rtype: dict
+        """ 
+
+        ReqBody = self.create_req_body()
+        UcfeReq = self.create_notification_ucfe_req(message_type=MESSAGES_TYPES.get('discard_notification'), 
+                                                    notification_type=notification_type, id_req=id_req)
+
+        return self.__send(ReqBody, UcfeReq)
+
     def get_document_report(self):
 
         conn_obj = conn.Connector()
@@ -367,7 +427,10 @@ class Manager(object):
         SubElement(Totales, "TpoMoneda").text = self.comprobante.currency or ''
         if self.comprobante.currency_rate: SubElement(Totales, "TpoCambio").text = self._format_tipo_cambio(self.comprobante.currency_rate)
         SubElement(Totales, "MntTotRetenido").text = self._format_amount(self.get_total_retention() * factor)
-        SubElement(Totales, "CantLinDet").text = '1'
+        detail_count = 0
+        for retention in self.comprobante.retentions:
+            detail_count += len(retention.retention_detail)
+        SubElement(Totales, "CantLinDet").text = str(detail_count)
         retentions = {}
         ret_nro = 0
         for ret in self.comprobante.retentions:
@@ -510,18 +573,16 @@ class Manager(object):
             SubElement(Item[item_nro], "MontoItem").text = self._format_amount(abs(line.amount))
 
     def detalle_eResg(self, Detalle):
-
-        Item = {0: SubElement(Detalle, "Item")}
-
-        if self.comprobante.retentions:
-            SubElement(Item[0], "NroLinDet").text = '1'
-
-        if hasattr(self.comprobante, 'refund_info'):
-            SubElement(Item[0], "IndFact").text = str(INVOICE_INDICATOR.get('refund'))
-
+        item_nro = 0
+        Item = {}
         for retention in self.comprobante.retentions:
             for detail in retention.retention_detail:
-                RetencPercep = SubElement(Item[0], "RetencPercep")
+                item_nro += 1
+                Item[item_nro] = SubElement(Detalle, "Item")
+                SubElement(Item[item_nro], "NroLinDet").text = str(item_nro)
+                if hasattr(self.comprobante, 'refund_info'):
+                    SubElement(Item[item_nro], "IndFact").text = str(INVOICE_INDICATOR.get('refund'))
+                RetencPercep = SubElement(Item[item_nro], "RetencPercep")
                 SubElement(RetencPercep, "CodRet").text = str(retention.retention_code)
                 if hasattr(retention, 'percentage_retention'):
                     SubElement(RetencPercep, "Tasa").text = self._format_amount(retention.percentage_retention)
